# Download the necessary files
# Python-GI bindings, Matplotlib (64-bit, Python 3.4)
wget -O pygi.exe https://sourceforge.net/projects/pygobjectwin32/files/pygi-aio-3.24.1_rev1-setup_049a323fe25432b10f7e9f543b74598d4be74a39.exe
